@@ -43,20 +43,26 @@ class Layer_Bevel : public synfig::Layer_Composite
 {
 	SYNFIG_LAYER_MODULE_EXT
 private:
-	synfig::Real 	softness;
-	int				type;
-
-	synfig::Color	color1;
-	synfig::Color	color2;
-
-	synfig::Angle	angle;
-	synfig::Real		depth;
-
+	//!Parameter: (int) type of blur to use
+	ValueBase param_type;
+	//!Parameter: (synfig::Real) amount of blur
+	ValueBase param_softness;
+	//!Parameter: (synfig::Color) light color
+	ValueBase param_color1;
+	//!Parameter: (synfig::Color) dark color
+	ValueBase param_color2;
+	//!Parameter: (synfig::Angle) angle of the light source
+	ValueBase param_angle;
+	//!Parameter: (synfig::Real) depth of the bevel
+	ValueBase param_depth;
+	//!Parameter: (bool) use luma
+	ValueBase param_use_luma;
+	//!Parameter: (bool) solid
+	ValueBase param_solid;
+	
 	synfig::Vector	offset;
 	synfig::Vector	offset45;
 
-	bool use_luma;
-	bool solid;
 
 	void calc_offset();
 public:
@@ -69,7 +75,7 @@ public:
 	virtual Color get_color(Context context, const Point &pos)const;
 
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context, cairo_t *cr,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 
 	virtual synfig::Rect get_full_bounding_rect(Context context)const;
 	virtual Vocab get_param_vocab()const;
