@@ -41,6 +41,7 @@
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
 #include <synfig/transform.h>
+#include <synfig/cairo_renddesc.h>
 
 #include <synfig/curve_helper.h>
 
@@ -81,60 +82,54 @@ SYNFIG_LAYER_SET_CVS_ID(Layer_SphereDistort,"$Id$");
 
 /* === E N T R Y P O I N T ================================================= */
 
-Layer_SphereDistort::Layer_SphereDistort()
-:center(0,0),
-radius(1),
-percent(1.0),
-type(TYPE_NORMAL),
-clip(false)
+Layer_SphereDistort::Layer_SphereDistort():
+param_center(ValueBase(Vector(0,0))),
+param_radius(ValueBase(double(1))),
+param_amount(ValueBase(double(1))),
+param_type(ValueBase(int(TYPE_NORMAL))),
+param_clip(ValueBase(false))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 
 bool
 Layer_SphereDistort::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT_PLUS(center,sync());
-	IMPORT_PLUS(radius,sync());
-	IMPORT(type);
-	IMPORT_AS(percent,"amount");
-	IMPORT(clip);
+	IMPORT_VALUE_PLUS(param_center,sync());
+	IMPORT_VALUE_PLUS(param_radius,sync());
+	IMPORT_VALUE(param_type);
+	IMPORT_VALUE(param_amount);
+	IMPORT_VALUE(param_clip);
 
-	if(param=="percent")
-	{
-		if(dynamic_param_list().count("percent"))
-		{
-			connect_dynamic_param("amount",dynamic_param_list().find("percent")->second);
-			disconnect_dynamic_param("percent");
-			synfig::warning("Layer_SphereDistort::::set_param(): Updated valuenode connection to use the new \"amount\" parameter.");
-		}
-		else
-			synfig::warning("Layer_SphereDistort::::set_param(): The parameter \"segment_list\" is deprecated. Use \"bline\" instead.");
-	}
+	if(param=="percent" && param_amount.get_type()==value.get_type())
+		return set_param("amount", value);
 
-	return false;
+	return Layer::set_param(param,value);
 }
 
 ValueBase
 Layer_SphereDistort::get_param(const String &param)const
 {
-	EXPORT(center);
-	EXPORT(radius);
-	EXPORT(type);
-	EXPORT_AS(percent,"amount");
-	EXPORT(clip);
+	EXPORT_VALUE(param_center);
+	EXPORT_VALUE(param_radius);
+	EXPORT_VALUE(param_type);
+	EXPORT_VALUE(param_amount);
+	EXPORT_VALUE(param_clip);
+	if(param=="percent")
+		return get_param("amount");
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
 
-	return ValueBase();
+	return Layer::get_param(param);
 }
 
 void
 Layer_SphereDistort::sync()
 {
+// Remove me?
 }
 
 Layer::Vocab
@@ -306,6 +301,12 @@ inline Point sphtrans(const Point &p, const Point &center, const Real &radius,
 synfig::Layer::Handle
 Layer_SphereDistort::hit_check(synfig::Context context, const synfig::Point &pos)const
 {
+	Vector center=param_center.get(Vector());
+	double radius=param_radius.get(double());
+	double percent=param_amount.get(double());
+	int type=param_type.get(int());
+	bool clip=param_clip.get(bool());
+	
 	bool clipped;
 	Point point(sphtrans(pos,center,radius,percent,type,clipped));
 	if(clip && clipped)
@@ -316,6 +317,12 @@ Layer_SphereDistort::hit_check(synfig::Context context, const synfig::Point &pos
 Color
 Layer_SphereDistort::get_color(Context context, const Point &pos)const
 {
+	Vector center=param_center.get(Vector());
+	double radius=param_radius.get(double());
+	double percent=param_amount.get(double());
+	int type=param_type.get(int());
+	bool clip=param_clip.get(bool());
+
 	bool clipped;
 	Point point(sphtrans(pos,center,radius,percent,type,clipped));
 	if(clip && clipped)
@@ -324,7 +331,8 @@ Layer_SphereDistort::get_color(Context context, const Point &pos)const
 }
 
 #if 1
-bool Layer_SphereDistort::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+bool
+Layer_SphereDistort::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
 	/*	Things to consider:
 		1) Block expansion for distortion (ouch... quality level??)
@@ -337,6 +345,11 @@ bool Layer_SphereDistort::accelerated_render(Context context,Surface *surface,in
 	*/
 
 	//bounding box reject
+	Vector center=param_center.get(Vector());
+	double radius=param_radius.get(double());
+	double percent=param_amount.get(double());
+	int type=param_type.get(int());
+	bool clip=param_clip.get(bool());
 	{
 		Rect	sphr;
 
@@ -532,6 +545,249 @@ bool Layer_SphereDistort::accelerated_render(Context context,Surface *surface,in
 
 	return true;
 }
+
+////////
+bool
+Layer_SphereDistort::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc_, ProgressCallback *cb)const
+{
+	/*	Things to consider:
+	 1) Block expansion for distortion (ouch... quality level??)
+	 2) Bounding box clipping
+	 3) Super sampling for better visual quality (based on the quality level?)
+	 4) Interpolation type for sampling (based on quality level?)
+	 
+	 //things to defer until after
+	 super sampling, non-linear interpolation
+	 */
+	
+	Vector center=param_center.get(Vector());
+	double radius=param_radius.get(double());
+	double percent=param_amount.get(double());
+	int type=param_type.get(int());
+	bool clip=param_clip.get(bool());
+
+	RendDesc	renddesc(renddesc_);
+	
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, renddesc))
+		return false;
+	
+	//bounding box reject
+	{
+		Rect	sphr;
+		
+		sphr.set_point(center[0]-radius,center[1]-radius);
+		sphr.expand(center[0]+radius,center[1]+radius);
+		
+		//get the bounding box of the transform
+		Rect	windr;
+		
+		//and the bounding box of the rendering
+		windr.set_point(renddesc.get_tl()[0],renddesc.get_tl()[1]);
+		windr.expand(renddesc.get_br()[0],renddesc.get_br()[1]);
+		
+		//test bounding boxes for collision
+		if( (type == TYPE_NORMAL && !intersect(sphr,windr)) ||
+		   (type == TYPE_DISTH && (sphr.minx >= windr.maxx || windr.minx >= sphr.maxx)) ||
+		   (type == TYPE_DISTV && (sphr.miny >= windr.maxy || windr.miny >= sphr.maxy)) )
+		{
+			if (clip)
+			{
+				cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+				cairo_paint(cr);
+				return true;
+			}
+			else
+				return context.accelerated_cairorender(cr,quality,renddesc,cb);
+		}
+	}
+	
+	//Ok, so we overlap some... now expand the window for rendering
+	RendDesc r = renddesc;
+	cairo_surface_t* background, *result;
+	int nl=0,nt=0,nr=0,nb=0, nw=0,nh=0;
+	
+	// grab the current renddesc interesting values
+	const Real pw = renddesc.get_pw(),ph = renddesc.get_ph();
+	const Point tl = renddesc.get_tl(), br = renddesc.get_br();
+	const int w = renddesc.get_w(), h = renddesc.get_h();
+	
+	{
+		//must enlarge window by pixel coordinates so go!
+		
+		//need to figure out closest and farthest point and distort THOSE
+		
+		Point origin[4] = {tl,tl,br,br};
+		Vector v[4] = {Vector(0,br[1]-tl[1]),
+			Vector(br[0]-tl[0],0),
+			Vector(0,tl[1]-br[1]),
+			Vector(tl[0]-br[0],0)};
+		
+		Point close(0,0);
+		Real t = 0;
+		Rect	expandr(tl,br);
+		
+		//expandr.set_point(tl[0],tl[1]);
+		//expandr.expand(br[0],br[1]);
+		
+		for(int i=0; i<4; ++i)
+		{
+			Vector p_o = center-origin[i];
+			
+			//project onto left line
+			t = (p_o*v[i])/v[i].mag_squared();
+			
+			//clamp
+			if(t < 0) t = 0; if(t > 1) t = 1;
+			
+			close = origin[i] + v[i]*t;
+			
+			//now get transforms and expand the rectangle to accommodate
+			Point p = sphtrans(close,center,radius,percent,type);
+			expandr.expand(p[0],p[1]);
+			p = sphtrans(origin[i],center,radius,percent,type);
+			expandr.expand(p[0],p[1]);
+			p = sphtrans(origin[i]+v[i],center,radius,percent,type);
+			expandr.expand(p[0],p[1]);
+		}
+		
+		//now that we have the bounding rectangle of ALL the pixels (should be...)
+		//order it so that it's in the same orientation as the tl,br pair
+		
+		Point ntl(0,0),nbr(0,0);
+		
+		//sort x
+		if(tl[0] < br[0])
+		{
+			ntl[0] = expandr.minx;
+			nbr[0] = expandr.maxx;
+		}
+		else
+		{
+			ntl[0] = expandr.maxx;
+			nbr[0] = expandr.minx;
+		}
+		
+		//sort y
+		if(tl[1] < br[1])
+		{
+			ntl[1] = expandr.miny;
+			nbr[1] = expandr.maxy;
+		}
+		else
+		{
+			ntl[1] = expandr.maxy;
+			nbr[1] = expandr.miny;
+		}
+		
+		//now expand the window as needed
+		Vector temp = ntl-tl;
+		
+		//pixel offset
+		nl = (int)(temp[0]/pw)-1;
+		nt = (int)(temp[1]/ph)-1;
+		
+		temp = nbr - br;
+		nr = (int)(temp[0]/pw)+1;
+		nb = (int)(temp[1]/ph)+1;
+		
+		nw = renddesc.get_w() + nr - nl;
+		nh = renddesc.get_h() + nb - nt;
+		
+		r.set_subwindow(nl,nt,nw,nh);
+	}
+	// New values for expanded
+	const double wpw =r.get_pw();
+	const double wph =r.get_ph();
+	const double wtlx=r.get_tl()[0];
+	const double wtly=r.get_tl()[1];
+
+	// now we know the sice of the needed background, create the cairo surface
+	background=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, nw, nh);
+	result=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, w, h);
+	// render the background
+	cairo_t* subcr=cairo_create(background);
+	cairo_scale(subcr, 1/wpw, 1/wph);
+	cairo_translate(subcr, -wtlx, -wtly);
+	if(!context.accelerated_cairorender(subcr,quality,r,cb))
+	{
+		synfig::warning("Cairo SphereDistort: Layer below failed");
+		return false;
+	}
+	cairo_destroy(subcr);
+	//now distort and check to make sure we aren't overshooting our bounds here
+	
+	Point sample = tl, sub = tl, trans(0,0);
+	float xs = 0,ys = 0;
+	int y=0,x=0;
+	Real invpw = 1/pw, invph = 1/ph;
+	Point rtl = r.get_tl();
+	
+	CairoSurface cresult(result);
+	if(!cresult.map_cairo_image())
+	{
+		synfig::warning("Sphere Distort: map cairo surface failed");
+		return false;
+	}
+	CairoSurface cbackground(background);
+	if(!cbackground.map_cairo_image())
+	{
+		synfig::warning("Sphere Distort: map cairo surface failed");
+		return false;
+	}
+	
+	for(y = 0; y < h; ++y, sample[1] += ph)
+	{
+		sub = sample;
+		for(x = 0; x < w; ++x, sub[0] += pw)
+		{
+			bool clipped;
+			trans=sphtrans(sub,center,radius,percent,type,clipped);
+			if(clip && clipped)
+			{
+				cresult[y][x]=CairoColor::alpha();
+				continue;
+			}
+			
+			xs = (trans[0]-rtl[0])*invpw;
+			ys = (trans[1]-rtl[1])*invph;
+			
+			if(!(xs >= 0 && xs < nw && ys >= 0 && ys < nh))
+			{
+				cresult[y][x]=context.get_cairocolor(trans).premult_alpha();
+				continue;
+			}
+			
+			//sample at that pixel location based on the quality
+			if(quality <= 4)	// cubic
+				cresult[y][x]=cbackground.cubic_sample_cooked(xs, ys).premult_alpha();
+			else if(quality <= 5) // cosine
+				cresult[y][x]=cbackground.cosine_sample_cooked(xs, ys).premult_alpha();
+			else if(quality <= 6) // linear
+				cresult[y][x]=cbackground.linear_sample_cooked(xs, ys).premult_alpha();
+			else				// nearest
+				cresult[y][x]=cbackground[round_to_int(ys)][round_to_int(xs)].premult_alpha();
+		}
+	}
+	cresult.unmap_cairo_image();
+	cbackground.unmap_cairo_image();
+	
+	cairo_surface_destroy(background);
+	
+	cairo_save(cr);
+	
+	cairo_translate(cr, tl[0], tl[1]);
+	cairo_scale(cr, pw, ph);
+	cairo_set_source_surface(cr, result, 0, 0);
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_paint(cr);
+	
+	cairo_restore(cr);
+	cairo_surface_destroy(result);
+	
+	return true;
+}
+
 #endif
 
 class synfig::Spherize_Trans : public synfig::Transform
@@ -542,12 +798,17 @@ public:
 
 	synfig::Vector perform(const synfig::Vector& x)const
 	{
-		return sphtrans(x,layer->center,layer->radius,-layer->percent,layer->type);
+		return sphtrans(x,layer->param_center.get(Vector()),layer->param_radius.get(double()),-layer->param_amount.get(double()),layer->param_type.get(int()));
 	}
 
 	synfig::Vector unperform(const synfig::Vector& x)const
 	{
-		return sphtrans(x,layer->center,layer->radius,layer->percent,layer->type);
+		return sphtrans(x,layer->param_center.get(Vector()),layer->param_radius.get(double()),-layer->param_amount.get(double()),layer->param_type.get(int()));
+	}
+
+	synfig::String get_string()const
+	{
+		return "spheredistort";
 	}
 };
 
@@ -560,6 +821,11 @@ Layer_SphereDistort::get_transform()const
 Rect
 Layer_SphereDistort::get_bounding_rect()const
 {
+	Vector center=param_center.get(Vector());
+	double radius=param_radius.get(double());
+	int type=param_type.get(int());
+	bool clip=param_clip.get(bool());
+
 	Rect bounds(Rect::full_plane());
 
 	if (clip)
