@@ -87,8 +87,7 @@ dv_trgt::dv_trgt(const char *Filename,
 	buffer=NULL;
 	wide_aspect=false;
 	color_buffer=0;
-		set_remove_alpha();
-
+	set_alpha_mode(TARGET_ALPHA_MODE_FILL);
 }
 
 dv_trgt::~dv_trgt()
