@@ -45,15 +45,16 @@ class Rectangle : public synfig::Layer_Composite, public synfig::Layer_NoDeform
 	SYNFIG_LAYER_MODULE_EXT
 
 private:
-
-	synfig::Color color;
-
-	synfig::Point point1;
-	synfig::Point point2;
-
-	synfig::Real expand;
-
-	bool invert;
+	//! Parameter: (Color)
+	synfig::ValueBase param_color;
+	//! Parameter: (Point)
+	synfig::ValueBase param_point1;
+	//! Parameter: (Point)
+	synfig::ValueBase param_point2;
+	//! Parameter: (Real)
+	synfig::ValueBase param_expand;
+	//! Parameter: (bool)
+	synfig::ValueBase param_invert;
 
 public:
 
@@ -68,7 +69,7 @@ public:
 	virtual synfig::Color get_color(synfig::Context context, const synfig::Point &pos)const;
 
 	virtual bool accelerated_render(synfig::Context context,synfig::Surface *surface,int quality, const synfig::RendDesc &renddesc, synfig::ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(synfig::Context context, cairo_surface_t *surface,int quality, const synfig::RendDesc &renddesc, synfig::ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(synfig::Context context, cairo_t *cr, int quality, const synfig::RendDesc &renddesc, synfig::ProgressCallback *cb)const;
 
 	synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
 
