@@ -43,6 +43,7 @@
 #include <synfig/surface.h>
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
+#include <synfig/cairo_renddesc.h>
 #include <ETL/bezier>
 #include <ETL/hermite>
 #include <ETL/calculus>
@@ -120,7 +121,7 @@ find_closest(bool fast, const std::vector<synfig::BLinePoint>& bline,const Point
 	next=bline.begin();
 
 	float best_bline_dist(0);
-	float best_bline_len(0);
+	//float best_bline_len(0);
 	float total_bline_dist(0);
 	float best_pos(0);
 	etl::hermite<Vector> best_curve;
@@ -156,7 +157,7 @@ find_closest(bool fast, const std::vector<synfig::BLinePoint>& bline,const Point
 
 		if (fast)
 		{
-#define POINT_CHECK(x) bp=curve(x);	thisdist=(bp-p).mag_squared(); if(thisdist<dist) { ret=iter; dist=thisdist; best_bline_dist=total_bline_dist; best_bline_len=len; best_curve=curve; }
+#define POINT_CHECK(x) bp=curve(x);	thisdist=(bp-p).mag_squared(); if(thisdist<dist) { ret=iter; dist=thisdist; best_bline_dist=total_bline_dist; /* best_bline_len=len; */ best_curve=curve; }
 			POINT_CHECK(0.0001);
 			POINT_CHECK((1.0/6.0));
 			POINT_CHECK((2.0/6.0));
@@ -174,7 +175,7 @@ find_closest(bool fast, const std::vector<synfig::BLinePoint>& bline,const Point
 				ret=iter;
 				dist=thisdist;
 				best_bline_dist=total_bline_dist;
-				best_bline_len=len;
+				//best_bline_len=len;
 				best_curve=curve;
 				best_pos = pos;
 			}
@@ -201,20 +202,23 @@ find_closest(bool fast, const std::vector<synfig::BLinePoint>& bline,const Point
 inline void
 CurveGradient::sync()
 {
+	std::vector<synfig::BLinePoint> bline(param_bline.get_list().begin(), param_bline.get_list().end());
 	curve_length_=calculate_distance(bline, bline_loop);
 }
 
 
 CurveGradient::CurveGradient():
 	Layer_Composite(1.0,Color::BLEND_COMPOSITE),
-	origin(0,0),
-	width(0.25),
-	gradient(Color::black(), Color::white()),
-	loop(false),
-	zigzag(false),
-	perpendicular(false),
-	fast(true)
+	param_origin(ValueBase(Point(0,0))),
+	param_width(ValueBase(Real(0.25))),
+	param_bline(ValueBase(std::vector<synfig::BLinePoint>())),
+	param_gradient(Gradient(Color::black(), Color::white())),
+	param_loop(ValueBase(false)),
+	param_zigzag(ValueBase(false)),
+	param_perpendicular(ValueBase(false)),
+	param_fast(ValueBase(true))
 {
+	std::vector<synfig::BLinePoint> bline;
 	bline.push_back(BLinePoint());
 	bline.push_back(BLinePoint());
 	bline.push_back(BLinePoint());
@@ -228,15 +232,26 @@ CurveGradient::CurveGradient():
 	bline[1].set_width(1.0f);
 	bline[2].set_width(1.0f);
 	bline_loop=true;
+	param_bline.set(bline);
 
 	sync();
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 inline Color
 CurveGradient::color_func(const Point &point_, int quality, float supersample)const
 {
+	Point origin=param_origin.get(Point());
+	Real width=param_width.get(Real());
+	std::vector<synfig::BLinePoint> bline(param_bline.get_list().begin(), param_bline.get_list().end());
+	Gradient gradient=param_gradient.get(Gradient());
+	bool loop=param_loop.get(bool());
+	bool zigzag=param_zigzag.get(bool());
+	bool perpendicular=param_perpendicular.get(bool());
+	bool fast=param_fast.get(bool());
+
 	Vector tangent;
 	Vector diff;
 	Point p1;
@@ -487,25 +502,23 @@ CurveGradient::set_param(const String & param, const ValueBase &value)
 {
 
 
-	IMPORT(origin);
-	IMPORT(perpendicular);
-	IMPORT(fast);
-
+	IMPORT_VALUE(param_origin);
+	IMPORT_VALUE(param_width);
 	if(param=="bline" && value.get_type()==ValueBase::TYPE_LIST)
 	{
-		bline=value;
+		param_bline=value;
 		bline_loop=value.get_loop();
 		sync();
-
 		return true;
 	}
+	IMPORT_VALUE(param_gradient);
+	IMPORT_VALUE(param_loop);
+	IMPORT_VALUE(param_zigzag);
+	IMPORT_VALUE(param_perpendicular);
+	IMPORT_VALUE(param_fast);
 
-	IMPORT(width);
-	IMPORT(gradient);
-	IMPORT(loop);
-	IMPORT(zigzag);
-
-	IMPORT_AS(origin,"offset");
+	if(param=="offset")
+		return set_param("origin", value);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -513,14 +526,14 @@ CurveGradient::set_param(const String & param, const ValueBase &value)
 ValueBase
 CurveGradient::get_param(const String & param)const
 {
-	EXPORT(origin);
-	EXPORT(bline);
-	EXPORT(gradient);
-	EXPORT(loop);
-	EXPORT(zigzag);
-	EXPORT(width);
-	EXPORT(perpendicular);
-	EXPORT(fast);
+	EXPORT_VALUE(param_origin);
+	EXPORT_VALUE(param_width);
+	EXPORT_VALUE(param_bline);
+	EXPORT_VALUE(param_gradient);
+	EXPORT_VALUE(param_loop);
+	EXPORT_VALUE(param_zigzag);
+	EXPORT_VALUE(param_perpendicular);
+	EXPORT_VALUE(param_fast);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -628,3 +641,70 @@ CurveGradient::accelerated_render(Context context,Surface *surface,int quality,
 
 	return true;
 }
+
+////
+bool
+CurveGradient::accelerated_cairorender(Context context, cairo_t *cr,int quality, const RendDesc &renddesc_, ProgressCallback *cb)const
+{
+	RendDesc	renddesc(renddesc_);
+	
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, renddesc))
+		return false;
+	
+	Point pos;
+	const Real pw(renddesc.get_pw()),ph(renddesc.get_ph());
+	const Point tl(renddesc.get_tl());
+	const int w(renddesc.get_w());
+	const int h(renddesc.get_h());
+	
+	SuperCallback supercb(cb,0,9500,10000);
+	
+	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
+	{
+		cairo_save(cr);
+		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+		cairo_paint(cr);
+		cairo_restore(cr);
+	}
+	else
+	{
+		if(!context.accelerated_cairorender(cr,quality,renddesc,&supercb))
+			return false;
+		if(get_amount()==0)
+			return true;
+	}
+	
+	
+	int x,y;
+	cairo_surface_t *surface;
+	
+	surface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, w, h);
+	
+	CairoSurface csurface(surface);
+	if(!csurface.map_cairo_image())
+	{
+		synfig::warning("Curve Gradient: map cairo surface failed");
+		return false;
+	}
+	for(y=0,pos[1]=tl[1];y<h;y++,pos[1]+=ph)
+		for(x=0,pos[0]=tl[0];x<w;x++,pos[0]+=pw)
+			csurface[y][x]=CairoColor(color_func(pos,calc_supersample(pos,pw,ph))).premult_alpha();
+	csurface.unmap_cairo_image();
+	
+	// paint surface on cr
+	cairo_save(cr);
+	cairo_translate(cr, tl[0], tl[1]);
+	cairo_scale(cr, pw, ph);
+	cairo_set_source_surface(cr, surface, 0, 0);
+	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+	cairo_restore(cr);
+	
+	cairo_surface_destroy(surface);
+	// Mark our progress as finished
+	if(cb && !cb->amount_complete(10000,10000))
+		return false;
+	
+	return true;
+
+}
