@@ -28,7 +28,7 @@
 
 /* === H E A D E R S ======================================================= */
 
-#include "string_decl.h"
+#include "string.h"
 
 /* === M A C R O S ========================================================= */
 
