@@ -88,7 +88,7 @@ int threads = 1;
  */
 const char* allowed_video_codecs[] =
 {
-	"flv", "h263p", "huffyuv", "libtheora", "libx264",
+	"flv", "h263p", "huffyuv", "libtheora", "libx264", "libx264-lossless",
 	"mjpeg", "mpeg1video", "mpeg2video", "mpeg4", "msmpeg4",
 	"msmpeg4v1", "msmpeg4v2", "wmv1", "wmv2", NULL
 };
@@ -103,7 +103,8 @@ const char* allowed_video_codecs_description[] =
 	"H.263+ / H.263-1998 / H.263 version 2.",
 	"Huffyuv / HuffYUV.",
 	"libtheora Theora.",
-	"libx264 H.264 / AVC / MPEG-4 AVC / MPEG-4 part 10.",
+	"H.264 / AVC / MPEG-4 AVC.",
+	"H.264 / AVC / MPEG-4 AVC (LossLess).",
 	"MJPEG (Motion JPEG).",
 	"raw MPEG-1 video.",
 	"raw MPEG-2 video.",
@@ -202,6 +203,7 @@ int main(int ac, char* av[])
             ("verbose,v", verbosity_arg_desc, _("Output verbosity level"))
             ("quiet,q", _("Quiet mode (No progress/time-remaining display)"))
             ("benchmarks,b", _("Print benchmarks"))
+            ("extract-alpha,x", _("Extract alpha"))
             ;
 
         po::options_description po_misc(_("Misc options"));
@@ -299,7 +301,15 @@ int main(int ac, char* av[])
 		job = op.extract_job();
 		job.desc = job.canvas->rend_desc() = op.extract_renddesc(job.canvas->rend_desc());
 
-		job_list.push_front(job);
+		if (job.extract_alpha) {
+			job.alpha_mode = TARGET_ALPHA_MODE_REDUCE;
+			job_list.push_front(job);
+			job.alpha_mode = TARGET_ALPHA_MODE_EXTRACT;
+			job.outfilename = filename_sans_extension(job.outfilename)+"-alpha"+filename_extension(job.outfilename);
+			job_list.push_front(job);
+		} else {
+			job_list.push_front(job);
+		}
 
 		process_job_list(job_list, op.extract_targetparam());
 
